GRID_SIZE = 32768
VERTICAL_LG_OFFSET = 6 # for logic gates with only x offset
VERTICAL_LG_HALF_OFFSET = 3 # for logic gates with x and y offset
HORIZONTAL_LG_OFFSET = 7 # for logic gates with x and y offset
CONNECTION_PIN_VERTICAL_OFFSET = 0
CONNECTION_PIN_HORIZONTAL_OFFSET = -1
INPUTS_HORIZONTAL_OFFSET = 15
INPUTS_VERTICAL_OFFSET = 10
NOT_GATE_VERTICAL_OFFSET = -1
NOT_GATE_HORIZONTAL_OFFSET = 2
AND_GATE_VERTICAL_OFFSET = 1
GATES_INTERNAL_PIN_OFFSETS = [0, 2]
SWITCH_HORIZONTAL_OFFSET = 1
SCENE_SIZE = 380
