class DecoderModes:
    AND_OR = "AND"
    OR_AND = "OR"

class LogicGateTypes:
    AND = "Logic AND"
    OR = "Logic OR"
    NOT = "Logic NOT"
    XOR = "Logic XOR"
    NOR = "Logic NOR"
    NAND = "Logic NAND"
